<?php


/**
 * メイン処理クラス
 */
Class Proc {

	// -------------------
	// Object
	// -------------------
	
	/**
     * validateクラスのインスタンス
     */
	var $cls_validate;
	
	/**
     * urlのインスタンス
     */
	var $cls_url;


	// -------------------
	// Variable
	// -------------------
	
	/**
     * 調査対象URL
     */
	var $url;

	/**
     * 巡回ドメイン
     */
	var $domain;

	/**
     * 取得リンク数
     */
	var $linkcount;

	/**
     * 再帰巡回数
     */
	var $roundcount;

	/**
     * Basic認証：ID
     */
	var $id;

	/**
     * Basic認証：PASSWORD
     */
	var $pass;


	// -------------------
	// Method
	// -------------------
	
	/**
	 * シングルトン
	 */
	function &getSingleton( $url = '', $domain = '', $linkcount = 10, $roundcount = 5, $id = '', $pass = '' ) {
		static $obj;
		if (is_null($obj)){
			// インスタンス生成
			$obj = new Proc();
			// 初期化メソッド呼出し
			$obj -> init($url,$domain,$linkcount,$roundcount,$id,$pass);
		}
		return $obj;
	}

	/**
	 * 初期化処理
	 */
	function init( $url, $domain, $linkcount, $roundcount, $id, $pass ) {
		// ベーシック認証チェック
		if ( $id != '' && $pass != '' ) {
			$url = mb_ereg_replace("http:\/\/", 'http://' . $id . ':' . $pass . '@', $url);
			$url = mb_ereg_replace("https:\/\/", 'http://' . $id . ':' . $pass . '@', $url);
		}
		// URL取得
		$this -> url = $url;
		$this -> domain = $domain;
		$this -> linkcount = $linkcount;
		$this -> roundcount = $roundcount;
		$this -> id = $id;
		$this -> pass = $pass;
		// インスタンス生成
		$this -> cls_validate = Validate::getSingleton();
		$this -> cls_url = Url::getSingleton($url,$domain,$linkcount,$roundcount,$id,$pass);
	}

	/**
	 * 動作制御
	 */
	function exec() {

		if ( $this -> url != '' && $this -> domain != '' ) {
			// リンクを取得
			$links = $this -> cls_url -> getLinks( $this -> url, $this -> domain );
			// バリデート実行
			$results = $this -> cls_validate -> execValidate( $links );
			// 画面表示
			include('result.php');
			exit;
		} else {
			echo 'パラメータが足りません';
			exit;
		}

	}

}


/**
 * URL処理クラス
 */
Class Url {

	// -------------------
	// Object
	// -------------------
	
	/**
     * htmlのインスタンス
     */
	var $cls_html;

	// -------------------
	// Variable
	// -------------------

	/**
     * 調査対象URL
     */
	var $url;

	/**
     * 巡回ドメイン
     */
	var $domain;

	/**
     * 取得リンク数
     */
	var $linkcount;

	/**
     * 再帰巡回数
     */
	var $roundcount;

	/**
     * Basic認証：ID
     */
	var $id;

	/**
     * Basic認証：PASSWORD
     */
	var $pass;


	// -------------------
	// Method
	// -------------------
	
	/**
	 * シングルトン
	 */
	function &getSingleton( $url = '', $domain = '', $linkcount = 10, $roundcount = 5, $id = '', $pass = '' ) {
		static $obj;
		if (is_null($obj)){
			// インスタンス生成
			$obj = new Url();
			// 初期化メソッド呼出し
			$obj -> init($url, $domain, $linkcount, $roundcount, $id, $pass);
		}
		return $obj;
	}

	/**
	 * 初期化処理
	 */
	function init($url, $domain, $linkcount, $roundcount, $id, $pass) {
		// 情報取得
		$this -> url = $url;
		$this -> domain = $domain;
		$this -> linkcount = $linkcount;
		$this -> roundcount = $roundcount;
		$this -> id = $id;
		$this -> pass = $pass;
		// インスタンス生成
		$this -> cls_html = Html::getSingleton();
	}

	/**
	 * 動作制御
	 */
	function getLinks( $url = '', $domain = '' ) {
		global $getLinks,$loopCnt;

		// 宣言されていなければ初期化
		if ( !isset($links) || is_null($links) ) {
			$links = array();
		}

		// 制限 ----------------------
		if ( $loopCnt >= $this -> roundcount ) {
			return $links;
		}
		// ---------------------------

		// 開始URLを追加
		if ( !array_search($This -> url, $getLinks) ) {
			$getLinks[] = $This -> url;
			$links = $This -> url;
		}

		// リンクを取得
		$links = array_merge($links, $this -> getLink( $this -> cls_html -> getHTML( $url, $domain ), $domain ));

		foreach ( $links as $val ) {

			// 未取得のURLなら取得
			if ( !array_search($val, $getLinks) ) {
				$getLinks[] = $val;
			} else {
				continue;
			}

			// 制限 ----------------------
			if ( $loopCnt >= $this -> roundcount ) {
				return $links;
			}
			// ---------------------------

			// 再帰呼び出しして追加
			$links = array_merge($links, $this -> getLinks( $val, $domain ));

			// 制限 ----------------------
			$loopCnt ++;
			// ---------------------------

		}

		// 制限 ----------------------
		$loopCnt ++;
		// ---------------------------

		// 取得したリンクを返す
		return $links;

	}

	/**
	 * リンク取得
	 */
	function getLink( $html, $domain ) {
		global $linkCnt;

		$links = array();

		if ( $html != null && $html != '' ) {
			if (preg_match_all('/<a\b[^>]*?>/si', $html, $matches)) {
				foreach ($matches[0] as $value) {
					if (false !== $x = $this->get_html_attrib('href', $value)) {
						if ( preg_match('/' . $domain . '/', $x) && !preg_match('/#/', $x) ) {
						//if ( preg_match('/' . $domain . '/', $x) && !preg_match('/https:\/\//', $x) && !preg_match('/#/', $x) ) {
							// 制限 ----------------------
							if ( $linkCnt >= $this -> linkcount ) {
								return $links;
							}
							// ---------------------------
							// ベーシック認証チェック
							if ( $this -> id != '' && $this -> pass != '' ) {
								$x = mb_ereg_replace("http:\/\/", 'http://' . $this -> id . ':' . $this -> pass . '@', $x);
								$x = mb_ereg_replace("https:\/\/", 'http://' . $this -> id . ':' . $this -> pass . '@', $x);
							}
							$links[] = $x;
							// 制限 ----------------------
							$linkCnt ++;
							// ---------------------------
						} elseif ( !preg_match('/http:\/\//', $x) && !preg_match('/https:\/\//', $x) && !preg_match('/#/', $x) ) {

							if ( !preg_match('/^..\//', $x) ) {
								// 制限 ----------------------
								if ( $linkCnt >= $this -> linkcount ) {
									return $links;
								}
								// ---------------------------
								$x = mb_ereg_replace("\/$", '', $this -> url) . '/' . $x;
								$links[] = $x;
								// 制限 ----------------------
								$linkCnt ++;
								// ---------------------------
							}

						}
					}
				}
			}
		}

		return $links;

	}

	/**
	 * 指定した属性の値を取得
	 */
	function get_html_attrib($attrib, $tag) {
		$regex = "/[\s'\"]$attrib\s*=\s*([^\s'\">]+|'[^']+'|\"[^\"]+\")/si";
		return preg_match($regex, $tag, $matches)?
		preg_replace('/^\s*[\'"](.+)[\'"]\s*$/s', '$1', $matches[1]):
		false;
	}

}


/**
 * HTML処理クラス
 */
Class Html {

	// -------------------
	// Object
	// -------------------
	
    /**
     * htmlのインスタンス
     */
	var $cls_html;
	
	/**
     * urlのインスタンス
     */
	var $cls_url;

	// -------------------
	// Method
	// -------------------
	
	/**
	 * シングルトン
	 */
	function &getSingleton() {
		static $obj;
		if (is_null($obj)){
			// インスタンス生成
			$obj = new Html();
		}
		return $obj;
	}

	/**
	 * URLからHTMLを取得
	 */
	function getHTML( $url = '', $domain = '' ) {

		$html = '';

		// サーバ状態チェック
		if ( $this -> checkServerState( $domain ) ) {

			// オープン
			$fp = fopen($url, "r");
			if ($fp) {
				while (!feof($fp)) {
					$html = $html . fgets($fp);
				}
				fclose($fp);
			} else {
				echo 'ファイルが開けませんでした：'.$url."\n";
			}

		}

		return $html;

	}

	/**
	 * サーバ状態チェック
	 */
	function checkServerState( $domain = '' ){

		// サーバ接続チェック
		if ( $domain != null && $domain != '' ) {
			$fp = fsockopen($domain, 80, $errno, $errstr, 10);
			if ($fp == false) {
				return false;
			}
			fclose($fp);
		}

		// ファイルOpenチェック
		//if ( $url != null && $url != '' ) {
		//	$fo = @fopen($url, "r");
		//	if ($fo == false) {
		//		echo 'ファイルが開けませんでした：'.$url."\n";
		//		return false;
		//	}
		//}

		return true;

	}

}


/**
 * バリデイト処理クラス
 */
Class Validate {

	// -------------------
	// Method
	// -------------------
	
	/**
	 * シングルトン
	 */
	function &getSingleton( $url = '' ) {
		static $obj;
		if (is_null($obj)){
			// インスタンス生成
			$obj = new Validate();
		}
		return $obj;
	}

	/**
	 * バリデートの実行
	 */
	function execValidate( $links ) {

		$results = array();

		foreach ($links as $val) {
			$data = 'uri=' . $val . '&output=soap12';
			$results[] = $this -> httpSocketConnection('validator.w3.org', 'GET', 'http://validator.w3.org/check', $data);
		}

		return $results;

	}

	/**
	 * HTTPリクエスト送信とレスポンス取得
	 */
	function httpSocketConnection($host, $method, $path, $data) {

		$method = strtoupper($method);

		if ($method == "GET"){
		    $path.= '?'.$data;
		}   

		$filePointer = fsockopen($host, 80, $errorNumber, $errorString);

		if (!$filePointer){
			echo 'Failed opening http socket connection: '.$errorString.' ('.$errorNumber.')<br/>\n';
			return false;
		}

		$requestHeader = $method." ".$path."  HTTP/1.1\r\n";
		$requestHeader.= "Host: ".$host."\r\n";
		$requestHeader.= "User-Agent:      Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1) Gecko/20061010 Firefox/2.0\r\n";
		$requestHeader.= "Content-Type: application/x-www-form-urlencoded\r\n";

		if ($method == "POST"){
		    $requestHeader.= "Content-Length: ".strlen($data)."\r\n";
		}

		$requestHeader.= "Connection: close\r\n\r\n";

		if ($method == "POST"){
		    $requestHeader.= $data;
		}

		fwrite($filePointer, $requestHeader);

		$responseHeader = '';
		$responseContent = '';

		do{
		    $responseHeader.= fread($filePointer, 1);
		}

		while (!preg_match('/\\r\\n\\r\\n$/', $responseHeader));

		if (!strstr($responseHeader, "Transfer-Encoding: chunked")){
		    while (!feof($filePointer)){
				$responseContent.= fgets($filePointer, 128);
		    }
		}else{
		    while ($chunk_length = hexdec(fgets($filePointer))){
				$responseContentChunk = '';
				$read_length = 0;
				while ($read_length < $chunk_length){
					$responseContentChunk .= fread($filePointer, $chunk_length - $read_length);
					$read_length = strlen($responseContentChunk);
				}
				$responseContent.= $responseContentChunk;
				fgets($filePointer);
			}
		}

		return chop($responseContent);
	}

}


$getLinks = array();
$loopCnt = 0;
$linkCnt = 0;

$app = Proc::getSingleton($_POST['url'],$_POST['domain'],$_POST['link_count'],$_POST['round_count'],$_POST['auth_id'],$_POST['auth_password']);
$app -> exec();


?>